/*
 * Livewallpaper
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (C) 2012-2013 Maximilian Schnarr <Maximilian.Schnarr@googlemail.com>
 *
 */

#include "main.h"


static gint
lw_config_plugin_store_sort_func(GtkTreeModel *model,
                                 GtkTreeIter *a,
                                 GtkTreeIter *b,
                                 gpointer data)
{
	gint sa, sb;

	(void) data;

	/* Sort by sort key */
	gtk_tree_model_get(model, a, SORT_KEY_COL, &sa, -1);
	gtk_tree_model_get(model, b, SORT_KEY_COL, &sb, -1);

	if(sa < sb)
		return -1;
	else if(sa > sb)
		return 1;
	else
	{
		/* Sort alphabetically */
		gint result;
		gchar *name_a, *name_b;

		gtk_tree_model_get(model, a, NAME_COL, &name_a, -1);
		gtk_tree_model_get(model, b, NAME_COL, &name_b, -1);

		result = g_strcmp0(name_a, name_b);

		g_free(name_a);
		g_free(name_b);

		return result;
	}
}

void
lw_config_plugin_store_add_heading(GtkListStore *store,
                                   const gchar *str,
                                   gint pos)
{
	GtkTreeIter iter;
	gtk_list_store_append(store, &iter);
	gtk_list_store_set(store, &iter,
	                   TITLE_COL, str,
	                   HEADING_ROW_COL, TRUE,
	                   SORT_KEY_COL, pos,
	                   -1);
}

void
lw_config_plugin_store_add_plugin(GtkListStore *store,
                                  LwConfigPlugin *p,
                                  gint pos,
                                  gboolean toggle)
{
	GtkTreeIter iter;
	gtk_list_store_append(store, &iter);
	gtk_list_store_set(store, &iter,
	                   PLUGIN_COL, p,
	                   ICON_COL, lw_config_plugin_get_icon(p, 48),
	                   NAME_COL, lw_config_plugin_get_name(p),
	                   PLUGIN_ROW_COL, TRUE,
	                   TOGGLE_ROW_COL, toggle,
	                   SORT_KEY_COL, pos,
	                   -1);
}

static void
add_plugin(LwConfigPlugin *p, GtkListStore *store)
{
	lw_config_plugin_store_add_plugin(store, p, LW_PLUGIN_POS, TRUE);
}

void
lw_config_plugin_store_add_plugins(GtkListStore *store,
                                   GSList *list)
{
	g_slist_foreach(list, (GFunc) add_plugin, store);
}

GtkListStore*
lw_config_plugin_store_new()
{
	GtkListStore *store = gtk_list_store_new(NUM_COLS,
	                                         LW_CONFIG_TYPE_PLUGIN,	/* PLUGIN_COL */
	                                         GDK_TYPE_PIXBUF,		/* ICON_COL */
	                                         G_TYPE_STRING,			/* NAME_COL */
	                                         G_TYPE_BOOLEAN,		/* PLUGIN_ROW_COL */
	                                         G_TYPE_BOOLEAN,		/* TOGGLE_ROW_COL */
	                                         G_TYPE_STRING,			/* TITLE_COL */
	                                         G_TYPE_BOOLEAN,		/* HEADING_ROW_COL */
	                                         G_TYPE_INT);			/* SORT_KEY_COL */

	/* Enable custom sort function */
	gtk_tree_sortable_set_default_sort_func(GTK_TREE_SORTABLE(store),
	                                        lw_config_plugin_store_sort_func,
	                                        NULL, NULL);
	gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(store),
	                                     GTK_TREE_SORTABLE_DEFAULT_SORT_COLUMN_ID,
	                                     GTK_SORT_ASCENDING);

	lw_config_plugin_store_add_heading(store, _("Wallpapers"), PLUGIN_HEADING_POS);
	/*lw_config_plugin_store_add_heading(store, _("Additional Effects"), LW_PLUGIN_POS + 1);*/

	return store;
}

