/*
 * Livewallpaper
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (C) 2012-2013 Maximilian Schnarr <Maximilian.Schnarr@googlemail.com>
 *
 */

#ifndef _LW_CONFIG_PLUGIN_H_
#define _LW_CONFIG_PLUGIN_H_

G_BEGIN_DECLS

#define LW_CONFIG_TYPE_PLUGIN            (lw_config_plugin_get_type())
#define LW_CONFIG_PLUGIN(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj),   LW_CONFIG_TYPE_PLUGIN, LwConfigPlugin))
#define LW_CONFIG_IS_PLUGIN(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj),   LW_CONFIG_TYPE_PLUGIN))
#define LW_CONFIG_PLUGIN_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST    ((klass), LW_CONFIG_TYPE_PLUGIN, LwConfigPluginClass))
#define LW_CONFIG_IS_PLUGIN_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE    ((klass), LW_CONFIG_TYPE_PLUGIN))
#define LW_CONFIG_PLUGIN_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS  ((obj),   LW_CONFIG_TYPE_PLUGIN, LwConfigPluginClass))

typedef struct _LwConfigPlugin LwConfigPlugin;
typedef struct _LwConfigPluginClass LwConfigPluginClass;

typedef struct _LwConfigPluginPrivate LwConfigPluginPrivate;

struct _LwConfigPlugin
{
	GObject parent_instance;

	/*< private >*/
	LwConfigPluginPrivate *priv;
};

struct _LwConfigPluginClass
{
	GObjectClass parent_class;
};

GType lw_config_plugin_get_type(void);

LwConfigPlugin *lw_config_plugin_new(const gchar *filename,
                                     const gchar *module_dir,
                                     const gchar *data_dir);

const gchar *lw_config_plugin_get_id(LwConfigPlugin *self);
GdkPixbuf *lw_config_plugin_get_icon(LwConfigPlugin *self, guint size);
const gchar *lw_config_plugin_get_name(LwConfigPlugin *self);
const gchar *lw_config_plugin_get_description(LwConfigPlugin *self);

GSettings *lw_config_plugin_get_settings(LwConfigPlugin *self);
const gchar *lw_config_plugin_get_schema_id(LwConfigPlugin *self);

const gchar *lw_config_plugin_get_module_dir(LwConfigPlugin *self);
const gchar *lw_config_plugin_get_data_dir(LwConfigPlugin *self);

const gchar *lw_config_plugin_get_schema_file(LwConfigPlugin *self);
const gchar *lw_config_plugin_get_enum_file(LwConfigPlugin *self);

GtkAboutDialog *lw_config_plugin_create_about_dialog(LwConfigPlugin *self);
GtkWidget *lw_config_plugin_get_settings_widget(LwConfigPlugin *self);

G_END_DECLS

#endif /* _LW_CONFIG_PLUGIN_H_ */

