/*
 * Livewallpaper
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (C) 2012-2013 Maximilian Schnarr <Maximilian.Schnarr@googlemail.com>
 *
 */

#include "main.h"
#include "loader-internal.h"


typedef struct
{
	GSettings *settings;
	gchar *name;

	GtkSpinButton *lower;
	GtkSpinButton *upper;

	gfloat min, max;
} RangeSelector;

static void range_selector_get_value(GSettings *,gchar *, RangeSelector *);
static void range_selector_set_value(RangeSelector *);
static void range_selector_lower_value_changed(GtkSpinButton *, RangeSelector *);
static void range_selector_upper_value_changed(GtkSpinButton *, RangeSelector *);

GtkWidget*
range_selector_new(Key *k, GSettings *settings)
{
	gchar *signame = g_strdup_printf("changed::%s", k->name);
	RangeSelector *selector = g_slice_new(RangeSelector);
	GtkWidget *tmp, *box = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 6);

	selector->settings = settings;
	selector->name = g_strdup(k->name);

	tmp = gtk_label_new(_("from"));
	gtk_box_pack_start(GTK_BOX(box), tmp, FALSE, FALSE, 0);

	/* Lower endpoint */
	tmp = gtk_spin_button_new_with_range(k->min, k->max, k->inc);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(tmp), k->min);
	gtk_spin_button_set_digits(GTK_SPIN_BUTTON(tmp), k->digits);
	g_signal_connect(tmp, "value-changed", G_CALLBACK(range_selector_lower_value_changed), selector);
	gtk_widget_set_size_request(tmp, 100, 0);
	gtk_box_pack_start(GTK_BOX(box), tmp, FALSE, FALSE, 0);
	selector->lower = GTK_SPIN_BUTTON(tmp);

	tmp = gtk_label_new(_("to"));
	gtk_box_pack_start(GTK_BOX(box), tmp, FALSE, FALSE, 0);

	/* Upper endpoint */
	tmp = gtk_spin_button_new_with_range(k->min, k->max, k->inc);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(tmp), k->max);
	gtk_spin_button_set_digits(GTK_SPIN_BUTTON(tmp), k->digits);
	g_signal_connect(tmp, "value-changed", G_CALLBACK(range_selector_upper_value_changed), selector);
	gtk_widget_set_size_request(tmp, 100, 0);
	gtk_box_pack_start(GTK_BOX(box), tmp, FALSE, FALSE, 0);
	selector->upper = GTK_SPIN_BUTTON(tmp);

	selector->min = k->min;
	selector->max = k->max;

	/* Get changes */
	g_signal_connect(settings, signame, G_CALLBACK(range_selector_get_value), selector);
	range_selector_get_value(NULL, NULL, selector);
	g_free(signame);

	return box;
}

static void
range_selector_get_value(GSettings *s, gchar *key, RangeSelector *selector)
{
	gdouble lower, upper;
	(void) s;
	(void) key;

	g_settings_get(selector->settings, selector->name, "(dd)",
	               &lower, &upper);

	gtk_spin_button_set_value(selector->lower, lower);
	gtk_spin_button_set_value(selector->upper, upper);
}

static void
range_selector_set_value(RangeSelector *selector)
{
	g_settings_set(selector->settings, selector->name, "(dd)",
	               gtk_spin_button_get_value(selector->lower),
	               gtk_spin_button_get_value(selector->upper));
}

static void
range_selector_lower_value_changed(GtkSpinButton *lower, RangeSelector *selector)
{
	gtk_spin_button_set_range(selector->upper, gtk_spin_button_get_value(lower), selector->max);
	range_selector_set_value(selector);
}

static void
range_selector_upper_value_changed(GtkSpinButton *upper, RangeSelector *selector)
{
	gtk_spin_button_set_range(selector->lower, selector->min, gtk_spin_button_get_value(upper));
	range_selector_set_value(selector);
}

