/*
 * Livewallpaper
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (C) 2012-2013 Maximilian Schnarr <Maximilian.Schnarr@gmx.de>
 */

#include "main.h"


#define KEY_FILE_HAS(key_file, key) g_key_file_has_key(key_file, "Plugin", key, NULL)

static GSList*
lw_indicator_find_plugins_real(gchar *module_dir, gchar *data_dir, guint recursions)
{
	GError *error = NULL;
	GSList *list = NULL;
	const gchar *dirent;
	GDir *d = g_dir_open(module_dir, 0, &error);

	if(error != NULL)
	{
		g_debug("Could not open module directory: %s", error->message);
		g_error_free(error);
		return NULL;
	}

	while((dirent = g_dir_read_name(d)) != NULL)
	{
		gchar *filename = g_build_filename(module_dir, dirent, NULL);

		if(g_file_test(filename, G_FILE_TEST_IS_DIR))
		{
			if(recursions > 0)
			{
				GSList *tmp = lw_indicator_find_plugins_real(filename,
				                                          data_dir,
				                                          recursions - 1);
				if(tmp != NULL)
					list = g_slist_concat(list, tmp);
			}
		}
		else if(g_str_has_suffix(dirent, ".plugin"))
		{
			GKeyFile *key_file = g_key_file_new();
			g_key_file_load_from_file(key_file, filename, G_KEY_FILE_NONE, &error);

			if(error != NULL)
			{
				g_critical("Could not load key file: %s", error->message);
				g_error_free(error);
				error = NULL;
			}
			else if(!KEY_FILE_HAS(key_file, "Module"))
			{
				g_critical("Could not get plugin's id from %s", filename);
				g_key_file_free(key_file);
			}
			else
				list = g_slist_prepend(list, key_file);
		}

		g_free(filename);
	}

	return list;
}

GSList*
lw_indicator_find_plugins(gchar *module_dir, gchar *data_dir)
{
	return lw_indicator_find_plugins_real(module_dir, data_dir, 1);
}

